"""Viewer modes - presentation and document rendering."""

from frago.viewer.modes.present import render_presentation
from frago.viewer.modes.document import render_document

__all__ = ["render_presentation", "render_document"]
