<!--
Purpose: Basic reveal.js presentation structure
Note: frago view automatically injects reveal.js library, no manual import needed
Preview: frago view slides.html --theme dracula
-->
<!DOCTYPE html>
<html>
<head>
    <meta charset="utf-8">
    <title>Presentation Title</title>
    <style>
        /* Global custom styles */
        .slides {
            /* Add global styles here */
        }
    </style>
</head>
<body>
    <div class="reveal">
        <div class="slides">

            <!-- Cover -->
            <section>
                <h1>Presentation Title</h1>
                <h3>Subtitle</h3>
                <p>Speaker | Date</p>
            </section>

            <!-- Table of Contents -->
            <section>
                <h2>Agenda</h2>
                <ul>
                    <li>Section 1</li>
                    <li>Section 2</li>
                    <li>Section 3</li>
                    <li>Summary</li>
                </ul>
            </section>

            <!-- Section 1 (with sub-pages) -->
            <section>
                <section>
                    <h2>Section 1 Title</h2>
                    <p>Section overview</p>
                </section>
                <section>
                    <h3>Details 1</h3>
                    <ul>
                        <li class="fragment">Point 1</li>
                        <li class="fragment">Point 2</li>
                        <li class="fragment">Point 3</li>
                    </ul>
                </section>
            </section>

            <!-- Section 2 -->
            <section>
                <h2>Section 2 Title</h2>
                <p>Content...</p>
            </section>

            <!-- Section 3 -->
            <section>
                <h2>Section 3 Title</h2>
                <pre><code class="language-python">
def example():
    return "Hello, World!"
                </code></pre>
            </section>

            <!-- Summary -->
            <section>
                <h2>Summary</h2>
                <ul>
                    <li class="fragment">Conclusion 1</li>
                    <li class="fragment">Conclusion 2</li>
                </ul>
                <p class="fragment"><strong>Thank You!</strong></p>
            </section>

        </div>
    </div>
</body>
</html>
